import { OnexConstants, OnomyConstants } from '@onomy/client';
import { OnomyHubConstants } from '@onomy/hub';

const DEVNET = {
  onomyChainInfo: OnomyConstants.CHAIN_INFO.TESTNET,
  onomyGraphqlEndpoint: 'https://api.thegraph.com/subgraphs/name/onomyprotocol/ograph-devnet',
  onexChainInfo: OnexConstants.CHAIN_INFO.DEVNET,
  onexGraphqlEndpoint: 'https://graphql-onex-devnet.onomy.io/subgraphs/name/onomyprotocol/mgraph',
  ethNetworkName: 'goerli',
  ethWsUrl: '',
  bNomContractAddress: '0x8EFe26D6839108E831D3a37cA503eA4F136A8E73',
  ethNomContractAddress: '0x9653a2013e23ae976d34543572e3e72f8578e4c6',
  bondContractAddress: '0xD179c5BEd30cADE4E62d53DD89240745Fb4C0CC2',
  gravityContractAddress: '0x18619DE15bDd14b0360e82e2746aAf77B17b3925',
  wEthUsdUniswapPoolAddress: '',
  ethBridgeWaitBlocks: 35,
};

const TESTNET = {
  onomyChainInfo: OnomyConstants.CHAIN_INFO.TESTNET,
  onomyGraphqlEndpoint: 'https://api.thegraph.com/subgraphs/name/onomyprotocol/ograph-testnet',
  onexChainInfo: OnexConstants.CHAIN_INFO.TESTNET,
  onexGraphqlEndpoint: 'https://graphql-onex-testnet.onomy.io/subgraphs/name/onomyprotocol/mgraph',
  ethNetworkName: 'goerli',
  ethWsUrl: '',
  bNomContractAddress: '0x8EFe26D6839108E831D3a37cA503eA4F136A8E73',
  ethNomContractAddress: '',
  bondContractAddress: '0xD179c5BEd30cADE4E62d53DD89240745Fb4C0CC2',
  gravityContractAddress: '0xBA7cDCD837757EB67Ea889d168Aba1a1Fa7C1340',
  wEthUsdUniswapPoolAddress: '',
  ethBridgeWaitBlocks: 64,
};

const MAINNET = {
  onomyChainInfo: OnomyConstants.CHAIN_INFO.MAINNET,
  onomyGraphqlEndpoint: 'https://api.thegraph.com/subgraphs/name/onomyprotocol/ograph-mainnet',
  onexChainInfo: undefined,
  onexGraphqlEndpoint: '',
  ethNetworkName: 'homestead',
  ethWsUrl: '',
  bNomContractAddress: '0x4fdF157C6860F33232608198419c74ED446Ca577',
  ethNomContractAddress: '',
  bondContractAddress: '0x264C13cfEd981e3137Fb43B198D14D8D5D64977E',
  gravityContractAddress: '0x85391fd149282C50B103aC810430F887685d575C',
  wEthUsdUniswapPoolAddress: '0x4e68ccd3e89f51c3074ca5072bbac773960dfa36',
  ethBridgeWaitBlocks: 64,
};

const OnomyEcosystemConfigs = {
  DEVNET,
  TESTNET,
  MAINNET,
};

export type OnomyEcosystemConfigName = keyof typeof OnomyEcosystemConfigs;
export type OnomyEcosystemConfig = typeof DEVNET | typeof TESTNET | typeof MAINNET;

export function setOnomyEcosystemConfig(
  configName: OnomyEcosystemConfigName,
  overrides?: Partial<OnomyEcosystemConfig>
) {
  const config = { ...OnomyEcosystemConfigs[configName], ...(overrides ?? {}) };
  OnomyHubConstants.updateAssetData({
    bNomAsset: {
      id: config.bNomContractAddress,
    },
    ethNomAsset: {
      id: config.ethNomContractAddress,
    },
    ethChainId: config.ethNetworkName,
  });

  return config;
}
