import { useCallback, useEffect, useMemo, useState } from 'react';
import BigNumber from 'bignumber.js';
import { useAsyncProcess } from '@onomy/react-utils';
import { OnexClient } from '@onomy/client';
import { useOnexClient } from '../hooks';

export function useRedeemDropForm({ dropId = '' } = {}) {
  const onex = useOnexClient();
  const [attemptTx, txStatus, resetTxStatus] = useAsyncProcess();
  const [txResult, setTxResult] = useState<null | Awaited<ReturnType<OnexClient['redeemDrop']>>>(
    null
  );

  const validationErrors = useMemo(() => {
    const errors: {
      dropId?: string;
    } = {};

    const bnDropId = new BigNumber(dropId);

    if (bnDropId.isNaN() || bnDropId.lte(0)) {
      errors.dropId = 'Coin A amount must be a number > 0';
    }

    return Object.keys(errors).length > 0 ? errors : null;
  }, [dropId]);

  const canSubmit = !validationErrors;

  const doRedeem = useCallback(
    (id?: string) => {
      resetTxStatus();
      setTxResult(null);
      return attemptTx(async () =>
        setTxResult((await onex?.redeemDrop({ uid: id ?? dropId })) ?? null)
      );
    },
    [onex, resetTxStatus, attemptTx, dropId]
  );

  const submitTx = useCallback(
    (e?: React.FormEvent) => {
      e?.preventDefault();
      if (!canSubmit) return Promise.resolve();
      return doRedeem(dropId);
    },
    [canSubmit, dropId, doRedeem]
  );

  const reset = useCallback(() => {
    resetTxStatus();
    setTxResult(null);
  }, [resetTxStatus]);

  useEffect(reset, [dropId, reset]);

  return {
    txStatus,
    txResult,

    canSubmit,
    validationErrors,

    dropId,

    doRedeem,
    submitTx,
    reset,
  };
}
